@@ -12,7 +12,6 @@ Required: Any = Ellipsis
 class ValidatorSignature(IntEnum):
     JUST_VALUE = 1
     VALUE_KWARGS = 2
-    BOUND_METHOD = 3
 
 
 class Shape(IntEnum):
@@ -23,7 +22,7 @@ class Shape(IntEnum):
 
 
 class Field:
-    __slots__ = ('type_', 'key_type_', 'sub_fields', 'key_field', 'validators', 'default', 'required',
+    __slots__ = ('type_', 'key_type_', 'sub_fields', 'key_field', 'validators', 'default', 'required', 'model_config',
                  'name', 'alias', 'description', 'info', 'validate_always', 'allow_none', 'shape', 'multipart')
 
     def __init__(
@@ -35,6 +34,7 @@ class Field:
             default: Any=None,
             required: bool=False,
             allow_none: bool=False,
+            model_config: Any=None,
             description: str=None):
 
         self.name: str = name
@@ -47,6 +47,7 @@ class Field:
         self.validators = []
         self.default: Any = default
         self.required: bool = required
+        self.model_config = model_config
         self.description: str = description
         self.allow_none: bool = allow_none
         self.shape: Shape = Shape.SINGLETON
@@ -55,8 +56,11 @@ class Field:
         self._prepare(class_validators or {})
 
     @classmethod
-    def infer(cls, *, name, value, annotation, class_validators, field_config):
+    def infer(cls, *, name, value, annotation, class_validators, config):
         required = value == Required
+        field_config = config.fields.get(name)
+        if isinstance(field_config, str):
+            field_config = {'alias': field_config}
         return cls(
             name=name,
             type_=annotation,
@@ -64,6 +68,7 @@ class Field:
             class_validators=class_validators,
             default=None if required else value,
             required=required,
+            model_config=config,
             description=field_config and field_config.get('description'),
         )
 
@@ -115,13 +120,14 @@ class Field:
                     self.allow_none = True
                 else:
                     types_.append(type_)
-            self.sub_fields = [Field(
+            self.sub_fields = [self.__class__(
                 type_=t,
                 class_validators=class_validators,
                 default=self.default,
                 required=self.required,
                 allow_none=self.allow_none,
-                name=f'{self.name}_{type_display(t)}'
+                name=f'{self.name}_{type_display(t)}',
+                model_config=self.model_config,
             ) for t in types_]
             self.multipart = True
         elif issubclass(origin, List):
@@ -135,24 +141,26 @@ class Field:
             self.key_type_ = self.type_.__args__[0]
             self.type_ = self.type_.__args__[1]
             self.shape = Shape.MAPPING
-            self.key_field = Field(
+            self.key_field = self.__class__(
                 type_=self.key_type_,
                 class_validators=class_validators,
                 default=self.default,
                 required=self.required,
                 allow_none=self.allow_none,
-                name=f'key_{self.name}'
+                name=f'key_{self.name}',
+                model_config=self.model_config,
             )
 
         if self.sub_fields is None and getattr(self.type_, '__origin__', False):
             self.multipart = True
-            self.sub_fields = [Field(
+            self.sub_fields = [self.__class__(
                 type_=self.type_,
                 class_validators=class_validators,
                 default=self.default,
                 required=self.required,
                 allow_none=self.allow_none,
-                name=f'_{self.name}'
+                name=f'_{self.name}',
+                model_config=self.model_config,
             )]
 
     def _populate_validators(self, class_validators):
@@ -173,29 +181,29 @@ class Field:
                 f,
             ))
 
-    def validate(self, v, model, index=None):
+    def validate(self, v, values, index=None):
         if self.allow_none and v is None:
             return None, None
 
         if self.shape is Shape.SINGLETON:
-            return self._validate_singleton(v, model, index)
+            return self._validate_singleton(v, values, index)
         elif self.shape is Shape.MAPPING:
-            return self._validate_mapping(v, model)
+            return self._validate_mapping(v, values)
         else:
             # list or set
-            result, errors = self._validate_sequence(v, model)
+            result, errors = self._validate_sequence(v, values)
             if not errors and self.shape is Shape.SET:
                 return set(result), errors
             return result, errors
 
-    def _validate_sequence(self, v, model):
+    def _validate_sequence(self, v, values):
         result, errors = [], []
         try:
             v_iter = enumerate(v)
         except TypeError as exc:
             return v, Error(exc, None, None)
         for i, v_ in v_iter:
-            single_result, single_errors = self._validate_singleton(v_, model, i)
+            single_result, single_errors = self._validate_singleton(v_, values, i)
             if single_errors:
                 errors.append(single_errors)
             else:
@@ -205,7 +213,7 @@ class Field:
         else:
             return result, None
 
-    def _validate_mapping(self, v, model):
+    def _validate_mapping(self, v, values):
         if isinstance(v, dict):
             v_iter = v
         else:
@@ -216,11 +224,11 @@ class Field:
 
         result, errors = {}, []
         for k, v_ in v_iter.items():
-            key_result, key_errors = self.key_field.validate(k, model, 'key')
+            key_result, key_errors = self.key_field.validate(k, values, 'key')
             if key_errors:
                 errors.append(key_errors)
                 continue
-            value_result, value_errors = self._validate_singleton(v_, model, k)
+            value_result, value_errors = self._validate_singleton(v_, values, k)
             if value_errors:
                 errors.append(value_errors)
                 continue
@@ -230,11 +238,11 @@ class Field:
         else:
             return result, None
 
-    def _validate_singleton(self, v, model, index):
+    def _validate_singleton(self, v, values, index):
         if self.multipart:
             errors = []
             for field in self.sub_fields:
-                value, error = field.validate(v, model, index)
+                value, error = field.validate(v, values, index)
                 if error:
                     errors.append(error)
                 else:
@@ -245,10 +253,9 @@ class Field:
                 try:
                     if signature is ValidatorSignature.JUST_VALUE:
                         v = validator(v)
-                    elif signature is ValidatorSignature.VALUE_KWARGS:
-                        v = validator(v, model=model, field=self)
                     else:
-                        v = validator(model, v)
+                        # ValidatorSignature.VALUE_KWARGS
+                        v = validator(v, values=values, config=self.model_config, field=self)
                 except (ValueError, TypeError) as exc:
                     return v, Error(exc, self.type_, index)
             return v, None
@@ -274,15 +281,12 @@ def _get_validator_signature(validator):
     # 1. we can deal with it before validation begins
     # 2. (more importantly) it doesn't get confused with a TypeError when executing the validator
     try:
-        if list(signature.parameters)[0] == 'self':
-            signature.bind(object(), 1)
-            return ValidatorSignature.BOUND_METHOD
-        elif len(signature.parameters) == 1:
+        if len(signature.parameters) == 1:
             signature.bind(1)
             return ValidatorSignature.JUST_VALUE
         else:
-            signature.bind(1, model=2, field=3)
+            signature.bind(1, values=2, config=3, field=4)
             return ValidatorSignature.VALUE_KWARGS
     except TypeError as e:
         raise ConfigError(f'Invalid signature for validator {validator}: {signature}, should be: '
-                          f'(value) or (value, *, model, field)') from e
+                          f'(value) or (value, *, values, config, field)') from e
