@@ -9,6 +9,7 @@ __all__ = [
     'NoneBytes',
     'StrBytes',
     'NoneStrBytes',
+    'StrictStr',
     'ConstrainedStr',
     'constr',
     'EmailStr',
@@ -27,6 +28,18 @@ StrBytes = Union[str, bytes]
 NoneStrBytes = Optional[StrBytes]
 
 
+class StrictStr(str):
+    @classmethod
+    def get_validators(cls):
+        yield cls.validate
+
+    @classmethod
+    def validate(cls, v):
+        if not isinstance(v, str):
+            raise ValueError(f'strict string: str expected not {type(v)}')
+        return v
+
+
 class ConstrainedStr(str):
     min_length: int = None
     max_length: int = None
@@ -83,6 +96,12 @@ class NameEmail:
     def validate(cls, value):
         return cls(*validate_email(value))
 
+    def __str__(self):
+        return f'{self.name} <{self.email}>'
+
+    def __repr__(self):
+        return f'<NameEmail("{self}")>'
+
 
 def constr(*, min_length=0, max_length=2**16, curtail_length=None, regex=None) -> Type[str]:
     # use kwargs then define conf in a dict to aid with IDE type hinting
@@ -114,6 +133,7 @@ class PyObject:
 
 class DSN(str):
     prefix = 'db_'
+    fields = 'driver', 'user', 'password', 'host', 'port', 'name', 'query'
     validate_always = True
 
     @classmethod
@@ -122,11 +142,10 @@ class DSN(str):
         yield cls.validate
 
     @classmethod
-    def validate(cls, value, model, **kwarg):
+    def validate(cls, value, values, **kwarg):
         if value:
             return value
-        d = model.__values__
-        kwargs = {f: d.get(cls.prefix + f) for f in ('driver', 'user', 'password', 'host', 'port', 'name', 'query')}
+        kwargs = {f: values.get(cls.prefix + f) for f in cls.fields}
         if kwargs['driver'] is None:
             raise ValueError(f'"{cls.prefix}driver" field may not be missing or None')
         return make_dsn(**kwargs)
