@@ -1,7 +1,9 @@
 from collections import OrderedDict
 from datetime import date, datetime, time, timedelta
+from decimal import Decimal
 from enum import Enum
 from pathlib import Path
+from typing import Any
 
 from .datetime_parse import parse_date, parse_datetime, parse_duration, parse_time
 from .exceptions import ConfigError
@@ -18,9 +20,13 @@ def not_none_validator(v):
 def str_validator(v) -> str:
     if isinstance(v, (str, NoneType)):
         return v
-    elif isinstance(v, bytes):
+    elif isinstance(v, (bytes, bytearray)):
         return v.decode()
-    return str(v)
+    elif isinstance(v, (float, int, Decimal)):
+        # is there anything else we want to add here? If you think so, create an issue.
+        return str(v)
+    else:
+        raise ValueError(f'str or byte type expected not {type(v)}')
 
 
 def bytes_validator(v) -> bytes:
@@ -47,16 +53,16 @@ def bool_validator(v) -> bool:
     return bool(v)
 
 
-def number_size_validator(v, model, **kwargs):
-    if model.config.min_number_size <= v <= model.config.max_number_size:
+def number_size_validator(v, config, **kwargs):
+    if config.min_number_size <= v <= config.max_number_size:
         return v
-    raise ValueError(f'size not in range {model.config.min_number_size} to {model.config.max_number_size}')
+    raise ValueError(f'size not in range {config.min_number_size} to {config.max_number_size}')
 
 
-def anystr_length_validator(v, model, **kwargs):
-    if v is None or model.config.min_anystr_length <= len(v) <= model.config.max_anystr_length:
+def anystr_length_validator(v, config, **kwargs):
+    if v is None or config.min_anystr_length <= len(v) <= config.max_anystr_length:
         return v
-    raise ValueError(f'length not in range {model.config.max_anystr_length} to {model.config.max_anystr_length}')
+    raise ValueError(f'length not in range {config.min_anystr_length} to {config.max_anystr_length}')
 
 
 def ordered_dict_validator(v) -> OrderedDict:
@@ -83,6 +89,12 @@ def tuple_validator(v) -> tuple:
     return tuple(v)
 
 
+def set_validator(v) -> set:
+    if isinstance(v, set):
+        return v
+    return set(v)
+
+
 def enum_validator(v, field, **kwargs) -> Enum:
     return field.type_(v)
 
@@ -109,11 +121,17 @@ _VALIDATORS = [
     (dict, [dict_validator]),
     (list, [list_validator]),
     (tuple, [tuple_validator]),
+    (set, [set_validator]),
 ]
 
 
 def find_validators(type_):
+    if type_ is Any:
+        return []
     for val_type, validators in _VALIDATORS:
-        if issubclass(type_, val_type):
-            return validators
+        try:
+            if issubclass(type_, val_type):
+                return validators
+        except TypeError as e:
+            raise TypeError(f'error checking inheritance of {type_!r} (type: {type(type_)})') from e
     raise ConfigError(f'no validator found for {type_}')
